module ActionView
  module Helpers
    module FormOptionsHelper
      def options_for_select(container, selected = nil)
        return container if String === container

        container = container.to_a if Hash === container
        selected, disabled = extract_selected_and_disabled(selected)

        options_for_select = container.inject([]) do |options, element|
          text, value = option_text_and_value(element)
          selected_attribute = ' selected="selected"' if option_value_selected?(value, selected)
          disabled_attribute = ' disabled="disabled"' if disabled && option_value_selected?(value, disabled)
          options << %(<option value="#{html_escape(value.to_s)}"#{selected_attribute}#{disabled_attribute}>#{html_escape(text.to_s)}</option>)
        end

        options_for_select.join("\n")
      end
      def extract_selected_and_disabled(selected)
          if selected.is_a?(Hash)
            [selected[:selected], selected[:disabled]]
          else
            [selected, nil]
          end
        end
      def grouped_options_for_select(grouped_options, selected_key = nil, prompt = nil)
        body = ''
        body << content_tag(:option, prompt, :value => "") if prompt
        grouped_options = grouped_options.sort if grouped_options.is_a?(Hash)
        grouped_options.each do |group|
          body << content_tag(:optgroup, options_for_select(group[1], selected_key), :label => group[0])
        end
        body
      end
    end
  end

  require 'action_view/test_case'
  require 'action_controller/test_process'
  class FormOptionsHelperTest < ActionView::TestCase
    def test_grouped_options_for_select_with_array
      assert_dom_equal(
            "<optgroup label=\"North America\"><option value=\"US\">United States</option>\n<option value=\"Canada\">Canada</option></optgroup><optgroup label=\"Europe\"><option value=\"GB\">Great Britain</option>\n<option value=\"Germany\">Germany</option></optgroup>",
            grouped_options_for_select([
               ["North America",
                       [['United States','US'],"Canada"]],
               ["Europe",
                       [["Great Britain","GB"], "Germany"]]
             ])
      )
    end

    def test_grouped_options_for_select_with_selected_and_prompt
      assert_dom_equal(
              "<option value=\"\">Choose a product...</option><optgroup label=\"Hats\"><option value=\"Baseball Cap\">Baseball Cap</option>\n<option selected=\"selected\" value=\"Cowboy Hat\">Cowboy Hat</option></optgroup>",
              grouped_options_for_select([["Hats", ["Baseball Cap","Cowboy Hat"]]], "Cowboy Hat", "Choose a product...")
      )
    end

    def test_optgroups_with_with_options_with_hash
      assert_dom_equal(
             "<optgroup label=\"Europe\"><option value=\"Denmark\">Denmark</option>\n<option value=\"Germany\">Germany</option></optgroup><optgroup label=\"North America\"><option value=\"United States\">United States</option>\n<option value=\"Canada\">Canada</option></optgroup>",
             grouped_options_for_select({'North America' => ['United States','Canada'], 'Europe' => ['Denmark','Germany']})
      )
    end
  end
end


